--
-- Default vars and utility functions
--
local filters = {}
local defaultFilterList = [[buying.+run[s%s%p]
selling.+run[s%s%p]
wt[bs].+run[s%s%p]
buying.+sto[nc]ks
selling.+sto[nc]ks
wt[bs].+sto[nc]ks
sto[nc]ks? spam
sto[nc]ks? run
buying.+carry
selling.+carry
wt[bs].+carry
selling.+mobs
wts.+mobs
1[%s-]*pull
one[%s-]*pull
mara.+pull
lf.+carry
carries
boo+st
wts.+gold
sell.+gold
#recruiting
#join.+guild
#wtb.+buff
#wtb.+port
#lf .+port
#buying.+port
#wtb.+summon
#lf .+summon
#buying.+summon]]
local colorHL = "|cffff55bb"
local colorReg = "|r"

local function trim(str)
    return str:gsub("^%s*(.-)%s*$", "%1")
end

local function loadFilters()
    filters = {}
    local i = 1
    for filter in STFU_Config.filters:gmatch("[^\r\n]+") do
        filter = trim(filter)
        -- Lines beginning with a # are comments
        if filter:sub(1, 1) ~= "#" then
            filters[i] = filter:lower()
            i = i + 1
        end
    end
end

--
-- Set up configuration options panel
--
local panel = CreateFrame("Frame", "STFU_ConfigPanel")
local title = panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge")
title:SetPoint("TOPLEFT", panel, "TOPLEFT", 11, -15)
title:SetText("Spam Text Filter Ultra " .. GetAddOnMetadata("STFU", "Version"))

local subtitle = panel:CreateFontString(nil, "ARTWORK", "GameFontNormal")
subtitle:SetPoint("TOPLEFT", title, "BOTTOMLEFT", 5, -10)
subtitle:SetTextColor(255/255, 255/255, 255/255)
subtitle:SetText("Enter keywords, phrases, or Lua patterns to filter public chat channels.")

local tips = panel:CreateFontString(nil, "ARTWORK", "GameFontNormal")
tips:SetPoint("TOPLEFT", subtitle, "BOTTOMLEFT", 0, -3)
tips:SetTextColor(180/255, 180/255, 100/255)
tips:SetJustifyH("LEFT")
tips:SetText(" • One pattern per line\n • Patterns are not case sensitive\n • Lines beginning with # are disabled")

local boxWidth = 500
local boxHeight = 350
local scrollWrapper = CreateFrame("Frame", nil, panel, BackdropTemplateMixin and "BackdropTemplate")
scrollWrapper:SetPoint("TOPLEFT", tips, "BOTTOMLEFT", 0, -10)
scrollWrapper:SetSize(boxWidth, boxHeight)
scrollWrapper:SetBackdrop({
    bgFile = "Interface/DialogFrame/UI-DialogBox-Background",
    edgeFile = "Interface/Tooltips/UI-Tooltip-Border",
    edgeSize = 16,
    insets = { left = 3, right = 3, top = 3, bottom = 3 }
})

local scroll = CreateFrame("ScrollFrame", "STFU_ScrollFrame", panel, "UIPanelScrollFrameTemplate")
scroll:SetPoint("TOPLEFT", scrollWrapper, "TOPLEFT", 12, -8)
scroll:SetSize(boxWidth - 16, boxHeight - 16)

local filterBox = CreateFrame("EditBox", nil, panel)
filterBox:SetPoint("TOPLEFT", scroll)
filterBox:SetPoint("BOTTOMRIGHT", scroll)
filterBox:SetSize(boxWidth - 24, boxHeight - 16)
filterBox:SetAutoFocus(false)
filterBox:SetMultiLine(true)
filterBox:SetFontObject("GameFontNormal")
filterBox:SetTextColor(200/255, 200/255, 200/255)
scroll:SetScrollChild(filterBox)

local debugToggle = CreateFrame("CheckButton", "STFU_DebugToggle", panel, "ChatConfigCheckButtonTemplate");
debugToggle:SetPoint("TOPLEFT", scrollWrapper, "BOTTOMLEFT", 0, -5)
STFU_DebugToggleText:SetText(" Debug mode (print filtered messages and invalid patterns in chat)")

panel.name = "STFU"
panel.okay = function(self)
    STFU_Config.filters = filterBox:GetText()
    STFU_Config.debugMode = debugToggle:GetChecked()
    loadFilters()
end
panel.cancel = function(self)
    filterBox:SetText(STFU_Config.filters)
    debugToggle:SetChecked(STFU_Config.debugMode)
end
panel.default = function(self)
    STFU_Config.filters = defaultFilterList
    filterBox:SetText(defaultFilterList)
    STFU_Config.debugMode = false
    debugToggle:SetChecked(STFU_Config.debugMode)
    loadFilters()
end
InterfaceOptions_AddCategory(panel)

--
-- Add a convenience slash command for opening the config panel
--
SLASH_STFU1 = "/stfu"
SlashCmdList.STFU = function(msg, editBox)
    InterfaceOptionsFrame_Show()
    InterfaceOptionsFrame_OpenToCategory("STFU")
end

--
-- Load saved filters
--
panel:RegisterEvent("ADDON_LOADED")
panel:RegisterEvent("CHAT_MSG_CHANNEL")
panel:SetScript("OnEvent", function(self, event, addon)
    if event == "ADDON_LOADED" and addon == "STFU" then
        if STFU_Config == nil then
            -- Use default filter list if this is the addon's first load
            STFU_Config.filters = defaultFilterList
            STFU_Config.debugMode = false
            print(colorHL .. "STFU: Thanks for choosing Spam Text Filter Ultra! You can add and remove chat filters by typing /stfu" .. colorReg)
        end
        filterBox:SetText(STFU_Config.filters)
        debugToggle:SetChecked(STFU_Config.debugMode)
        loadFilters()
    end
end)

--
-- Bind chat filter function
--
local lastMsg = ""
local lastResult = false

local function testMsgWithFilter(msg, filter)
    return msg:find(filter)
end

ChatFrame_AddMessageEventFilter("CHAT_MSG_CHANNEL", function(self, event, msg, author)
    local player, server = UnitFullName("player")
    if author == player .. "-" .. server then
        return false
    end
    msg = msg:lower()
    if msg == lastMsg then
        return lastResult
    end
    for i, filter in ipairs(filters) do
        local status, result = pcall(testMsgWithFilter, msg, filter)
        if STFU_Config.debugMode then
            if status == false then
                print(colorHL .. "INVALID PATTERN (line " .. (i + 1) .. "): " .. colorReg .. filter)
            elseif result then
                print(colorHL .. "FILTERED: " .. colorReg .. filter .. colorHL .. " => " .. colorReg .. msg:gsub("item:.-%[", "["))
            end
        end
        if status and result then
            lastMsg = msg
            lastResult = true
            return true
        end
    end
    lastMsg = msg
    lastResult = false
    return false
end)